/****************************************************************************
** Meta object code from reading C++ file 'camerabincontrol.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../git/src/plugins/gstreamer/camerabin/camerabincontrol.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'camerabincontrol.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CameraBinControl_t {
    QByteArrayData data[13];
    char stringdata0[214];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CameraBinControl_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CameraBinControl_t qt_meta_stringdata_CameraBinControl = {
    {
QT_MOC_LITERAL(0, 0, 16), // "CameraBinControl"
QT_MOC_LITERAL(1, 17, 11), // "reloadLater"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 33), // "setViewfinderColorSpaceConver..."
QT_MOC_LITERAL(4, 64, 7), // "enabled"
QT_MOC_LITERAL(5, 72, 13), // "delayedReload"
QT_MOC_LITERAL(6, 86, 22), // "handleResourcesGranted"
QT_MOC_LITERAL(7, 109, 19), // "handleResourcesLost"
QT_MOC_LITERAL(8, 129, 17), // "handleBusyChanged"
QT_MOC_LITERAL(9, 147, 17), // "handleCameraError"
QT_MOC_LITERAL(10, 165, 5), // "error"
QT_MOC_LITERAL(11, 171, 11), // "errorString"
QT_MOC_LITERAL(12, 183, 30) // "viewfinderColorSpaceConversion"

    },
    "CameraBinControl\0reloadLater\0\0"
    "setViewfinderColorSpaceConversion\0"
    "enabled\0delayedReload\0handleResourcesGranted\0"
    "handleResourcesLost\0handleBusyChanged\0"
    "handleCameraError\0error\0errorString\0"
    "viewfinderColorSpaceConversion"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CameraBinControl[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       1,   64, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x0a /* Public */,
       3,    1,   50,    2, 0x0a /* Public */,
       5,    0,   53,    2, 0x08 /* Private */,
       6,    0,   54,    2, 0x08 /* Private */,
       7,    0,   55,    2, 0x08 /* Private */,
       8,    1,   56,    2, 0x08 /* Private */,
       9,    2,   59,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   10,   11,

 // properties: name, type, flags
      12, QMetaType::Bool, 0x00095103,

       0        // eod
};

void CameraBinControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CameraBinControl *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->reloadLater(); break;
        case 1: _t->setViewfinderColorSpaceConversion((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->delayedReload(); break;
        case 3: _t->handleResourcesGranted(); break;
        case 4: _t->handleResourcesLost(); break;
        case 5: _t->handleBusyChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->handleCameraError((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<CameraBinControl *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->viewfinderColorSpaceConversion(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<CameraBinControl *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setViewfinderColorSpaceConversion(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject CameraBinControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QCameraControl::staticMetaObject>(),
    qt_meta_stringdata_CameraBinControl.data,
    qt_meta_data_CameraBinControl,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CameraBinControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CameraBinControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CameraBinControl.stringdata0))
        return static_cast<void*>(this);
    return QCameraControl::qt_metacast(_clname);
}

int CameraBinControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QCameraControl::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
