/****************************************************************************
** Meta object code from reading C++ file 'qgstreamerplayersession_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/gsttools/qgstreamerplayersession_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgstreamerplayersession_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QGstreamerPlayerSession_t {
    QByteArrayData data[52];
    char stringdata0[686];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QGstreamerPlayerSession_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QGstreamerPlayerSession_t qt_meta_stringdata_QGstreamerPlayerSession = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QGstreamerPlayerSession"
QT_MOC_LITERAL(1, 24, 15), // "durationChanged"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 8), // "duration"
QT_MOC_LITERAL(4, 50, 15), // "positionChanged"
QT_MOC_LITERAL(5, 66, 8), // "position"
QT_MOC_LITERAL(6, 75, 12), // "stateChanged"
QT_MOC_LITERAL(7, 88, 19), // "QMediaPlayer::State"
QT_MOC_LITERAL(8, 108, 5), // "state"
QT_MOC_LITERAL(9, 114, 13), // "volumeChanged"
QT_MOC_LITERAL(10, 128, 6), // "volume"
QT_MOC_LITERAL(11, 135, 17), // "mutedStateChanged"
QT_MOC_LITERAL(12, 153, 5), // "muted"
QT_MOC_LITERAL(13, 159, 21), // "audioAvailableChanged"
QT_MOC_LITERAL(14, 181, 14), // "audioAvailable"
QT_MOC_LITERAL(15, 196, 21), // "videoAvailableChanged"
QT_MOC_LITERAL(16, 218, 14), // "videoAvailable"
QT_MOC_LITERAL(17, 233, 24), // "bufferingProgressChanged"
QT_MOC_LITERAL(18, 258, 13), // "percentFilled"
QT_MOC_LITERAL(19, 272, 16), // "playbackFinished"
QT_MOC_LITERAL(20, 289, 11), // "tagsChanged"
QT_MOC_LITERAL(21, 301, 14), // "streamsChanged"
QT_MOC_LITERAL(22, 316, 15), // "seekableChanged"
QT_MOC_LITERAL(23, 332, 5), // "error"
QT_MOC_LITERAL(24, 338, 11), // "errorString"
QT_MOC_LITERAL(25, 350, 12), // "invalidMedia"
QT_MOC_LITERAL(26, 363, 19), // "playbackRateChanged"
QT_MOC_LITERAL(27, 383, 15), // "rendererChanged"
QT_MOC_LITERAL(28, 399, 15), // "pipelineChanged"
QT_MOC_LITERAL(29, 415, 11), // "loadFromUri"
QT_MOC_LITERAL(30, 427, 15), // "QNetworkRequest"
QT_MOC_LITERAL(31, 443, 3), // "url"
QT_MOC_LITERAL(32, 447, 14), // "loadFromStream"
QT_MOC_LITERAL(33, 462, 10), // "QIODevice*"
QT_MOC_LITERAL(34, 473, 6), // "stream"
QT_MOC_LITERAL(35, 480, 4), // "play"
QT_MOC_LITERAL(36, 485, 5), // "pause"
QT_MOC_LITERAL(37, 491, 4), // "stop"
QT_MOC_LITERAL(38, 496, 4), // "seek"
QT_MOC_LITERAL(39, 501, 3), // "pos"
QT_MOC_LITERAL(40, 505, 9), // "setVolume"
QT_MOC_LITERAL(41, 515, 8), // "setMuted"
QT_MOC_LITERAL(42, 524, 17), // "showPrerollFrames"
QT_MOC_LITERAL(43, 542, 7), // "enabled"
QT_MOC_LITERAL(44, 550, 14), // "getStreamsInfo"
QT_MOC_LITERAL(45, 565, 11), // "setSeekable"
QT_MOC_LITERAL(46, 577, 23), // "finishVideoOutputChange"
QT_MOC_LITERAL(47, 601, 19), // "updateVideoRenderer"
QT_MOC_LITERAL(48, 621, 24), // "updateVideoResolutionTag"
QT_MOC_LITERAL(49, 646, 12), // "updateVolume"
QT_MOC_LITERAL(50, 659, 11), // "updateMuted"
QT_MOC_LITERAL(51, 671, 14) // "updateDuration"

    },
    "QGstreamerPlayerSession\0durationChanged\0"
    "\0duration\0positionChanged\0position\0"
    "stateChanged\0QMediaPlayer::State\0state\0"
    "volumeChanged\0volume\0mutedStateChanged\0"
    "muted\0audioAvailableChanged\0audioAvailable\0"
    "videoAvailableChanged\0videoAvailable\0"
    "bufferingProgressChanged\0percentFilled\0"
    "playbackFinished\0tagsChanged\0"
    "streamsChanged\0seekableChanged\0error\0"
    "errorString\0invalidMedia\0playbackRateChanged\0"
    "rendererChanged\0pipelineChanged\0"
    "loadFromUri\0QNetworkRequest\0url\0"
    "loadFromStream\0QIODevice*\0stream\0play\0"
    "pause\0stop\0seek\0pos\0setVolume\0setMuted\0"
    "showPrerollFrames\0enabled\0getStreamsInfo\0"
    "setSeekable\0finishVideoOutputChange\0"
    "updateVideoRenderer\0updateVideoResolutionTag\0"
    "updateVolume\0updateMuted\0updateDuration"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QGstreamerPlayerSession[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      34,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      17,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  184,    2, 0x06 /* Public */,
       4,    1,  187,    2, 0x06 /* Public */,
       6,    1,  190,    2, 0x06 /* Public */,
       9,    1,  193,    2, 0x06 /* Public */,
      11,    1,  196,    2, 0x06 /* Public */,
      13,    1,  199,    2, 0x06 /* Public */,
      15,    1,  202,    2, 0x06 /* Public */,
      17,    1,  205,    2, 0x06 /* Public */,
      19,    0,  208,    2, 0x06 /* Public */,
      20,    0,  209,    2, 0x06 /* Public */,
      21,    0,  210,    2, 0x06 /* Public */,
      22,    1,  211,    2, 0x06 /* Public */,
      23,    2,  214,    2, 0x06 /* Public */,
      25,    0,  219,    2, 0x06 /* Public */,
      26,    1,  220,    2, 0x06 /* Public */,
      27,    0,  223,    2, 0x06 /* Public */,
      28,    0,  224,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      29,    1,  225,    2, 0x0a /* Public */,
      32,    2,  228,    2, 0x0a /* Public */,
      35,    0,  233,    2, 0x0a /* Public */,
      36,    0,  234,    2, 0x0a /* Public */,
      37,    0,  235,    2, 0x0a /* Public */,
      38,    1,  236,    2, 0x0a /* Public */,
      40,    1,  239,    2, 0x0a /* Public */,
      41,    1,  242,    2, 0x0a /* Public */,
      42,    1,  245,    2, 0x0a /* Public */,
      44,    0,  248,    2, 0x08 /* Private */,
      45,    1,  249,    2, 0x08 /* Private */,
      46,    0,  252,    2, 0x08 /* Private */,
      47,    0,  253,    2, 0x08 /* Private */,
      48,    0,  254,    2, 0x08 /* Private */,
      49,    0,  255,    2, 0x08 /* Private */,
      50,    0,  256,    2, 0x08 /* Private */,
      51,    0,  257,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::LongLong,    3,
    QMetaType::Void, QMetaType::LongLong,    5,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   23,   24,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QReal,    2,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 30,   31,
    QMetaType::Void, 0x80000000 | 30, 0x80000000 | 33,   31,   34,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::LongLong,   39,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::Bool,   43,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QGstreamerPlayerSession::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QGstreamerPlayerSession *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->durationChanged((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 1: _t->positionChanged((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 2: _t->stateChanged((*reinterpret_cast< QMediaPlayer::State(*)>(_a[1]))); break;
        case 3: _t->volumeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->mutedStateChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->audioAvailableChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->videoAvailableChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->bufferingProgressChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->playbackFinished(); break;
        case 9: _t->tagsChanged(); break;
        case 10: _t->streamsChanged(); break;
        case 11: _t->seekableChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->error((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 13: _t->invalidMedia(); break;
        case 14: _t->playbackRateChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 15: _t->rendererChanged(); break;
        case 16: _t->pipelineChanged(); break;
        case 17: _t->loadFromUri((*reinterpret_cast< const QNetworkRequest(*)>(_a[1]))); break;
        case 18: _t->loadFromStream((*reinterpret_cast< const QNetworkRequest(*)>(_a[1])),(*reinterpret_cast< QIODevice*(*)>(_a[2]))); break;
        case 19: { bool _r = _t->play();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->pause();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->stop(); break;
        case 22: { bool _r = _t->seek((*reinterpret_cast< qint64(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->setVolume((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 24: _t->setMuted((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 25: _t->showPrerollFrames((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 26: _t->getStreamsInfo(); break;
        case 27: _t->setSeekable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 28: _t->finishVideoOutputChange(); break;
        case 29: _t->updateVideoRenderer(); break;
        case 30: _t->updateVideoResolutionTag(); break;
        case 31: _t->updateVolume(); break;
        case 32: _t->updateMuted(); break;
        case 33: _t->updateDuration(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMediaPlayer::State >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QNetworkRequest >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QIODevice* >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QNetworkRequest >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QGstreamerPlayerSession::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::durationChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::positionChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)(QMediaPlayer::State );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::stateChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::volumeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::mutedStateChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::audioAvailableChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::videoAvailableChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::bufferingProgressChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::playbackFinished)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::tagsChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::streamsChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::seekableChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)(int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::error)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::invalidMedia)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::playbackRateChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::rendererChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QGstreamerPlayerSession::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGstreamerPlayerSession::pipelineChanged)) {
                *result = 16;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QGstreamerPlayerSession::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QGstreamerPlayerSession.data,
    qt_meta_data_QGstreamerPlayerSession,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QGstreamerPlayerSession::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGstreamerPlayerSession::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QGstreamerPlayerSession.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGstreamerBusMessageFilter"))
        return static_cast< QGstreamerBusMessageFilter*>(this);
    if (!strcmp(_clname, "org.qt-project.qt.gstreamerbusmessagefilter/5.0"))
        return static_cast< QGstreamerBusMessageFilter*>(this);
    return QObject::qt_metacast(_clname);
}

int QGstreamerPlayerSession::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
    return _id;
}

// SIGNAL 0
void QGstreamerPlayerSession::durationChanged(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QGstreamerPlayerSession::positionChanged(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QGstreamerPlayerSession::stateChanged(QMediaPlayer::State _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QGstreamerPlayerSession::volumeChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QGstreamerPlayerSession::mutedStateChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QGstreamerPlayerSession::audioAvailableChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QGstreamerPlayerSession::videoAvailableChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QGstreamerPlayerSession::bufferingProgressChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QGstreamerPlayerSession::playbackFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QGstreamerPlayerSession::tagsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QGstreamerPlayerSession::streamsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QGstreamerPlayerSession::seekableChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QGstreamerPlayerSession::error(int _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QGstreamerPlayerSession::invalidMedia()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QGstreamerPlayerSession::playbackRateChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QGstreamerPlayerSession::rendererChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QGstreamerPlayerSession::pipelineChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
