/****************************************************************************
** Meta object code from reading C++ file 'qradiodata.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/multimedia/radio/qradiodata.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qradiodata.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QRadioData_t {
    QByteArrayData data[74];
    char stringdata0[907];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QRadioData_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QRadioData_t qt_meta_stringdata_QRadioData = {
    {
QT_MOC_LITERAL(0, 0, 10), // "QRadioData"
QT_MOC_LITERAL(1, 11, 16), // "stationIdChanged"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 9), // "stationId"
QT_MOC_LITERAL(4, 39, 18), // "programTypeChanged"
QT_MOC_LITERAL(5, 58, 23), // "QRadioData::ProgramType"
QT_MOC_LITERAL(6, 82, 11), // "programType"
QT_MOC_LITERAL(7, 94, 22), // "programTypeNameChanged"
QT_MOC_LITERAL(8, 117, 15), // "programTypeName"
QT_MOC_LITERAL(9, 133, 18), // "stationNameChanged"
QT_MOC_LITERAL(10, 152, 11), // "stationName"
QT_MOC_LITERAL(11, 164, 16), // "radioTextChanged"
QT_MOC_LITERAL(12, 181, 9), // "radioText"
QT_MOC_LITERAL(13, 191, 36), // "alternativeFrequenciesEnabled..."
QT_MOC_LITERAL(14, 228, 7), // "enabled"
QT_MOC_LITERAL(15, 236, 5), // "error"
QT_MOC_LITERAL(16, 242, 17), // "QRadioData::Error"
QT_MOC_LITERAL(17, 260, 32), // "setAlternativeFrequenciesEnabled"
QT_MOC_LITERAL(18, 293, 19), // "_q_serviceDestroyed"
QT_MOC_LITERAL(19, 313, 11), // "ProgramType"
QT_MOC_LITERAL(20, 325, 29), // "alternativeFrequenciesEnabled"
QT_MOC_LITERAL(21, 355, 5), // "Error"
QT_MOC_LITERAL(22, 361, 7), // "NoError"
QT_MOC_LITERAL(23, 369, 13), // "ResourceError"
QT_MOC_LITERAL(24, 383, 9), // "OpenError"
QT_MOC_LITERAL(25, 393, 15), // "OutOfRangeError"
QT_MOC_LITERAL(26, 409, 9), // "Undefined"
QT_MOC_LITERAL(27, 419, 4), // "News"
QT_MOC_LITERAL(28, 424, 14), // "CurrentAffairs"
QT_MOC_LITERAL(29, 439, 11), // "Information"
QT_MOC_LITERAL(30, 451, 5), // "Sport"
QT_MOC_LITERAL(31, 457, 9), // "Education"
QT_MOC_LITERAL(32, 467, 5), // "Drama"
QT_MOC_LITERAL(33, 473, 7), // "Culture"
QT_MOC_LITERAL(34, 481, 7), // "Science"
QT_MOC_LITERAL(35, 489, 6), // "Varied"
QT_MOC_LITERAL(36, 496, 8), // "PopMusic"
QT_MOC_LITERAL(37, 505, 9), // "RockMusic"
QT_MOC_LITERAL(38, 515, 13), // "EasyListening"
QT_MOC_LITERAL(39, 529, 14), // "LightClassical"
QT_MOC_LITERAL(40, 544, 16), // "SeriousClassical"
QT_MOC_LITERAL(41, 561, 10), // "OtherMusic"
QT_MOC_LITERAL(42, 572, 7), // "Weather"
QT_MOC_LITERAL(43, 580, 7), // "Finance"
QT_MOC_LITERAL(44, 588, 19), // "ChildrensProgrammes"
QT_MOC_LITERAL(45, 608, 13), // "SocialAffairs"
QT_MOC_LITERAL(46, 622, 8), // "Religion"
QT_MOC_LITERAL(47, 631, 7), // "PhoneIn"
QT_MOC_LITERAL(48, 639, 6), // "Travel"
QT_MOC_LITERAL(49, 646, 7), // "Leisure"
QT_MOC_LITERAL(50, 654, 9), // "JazzMusic"
QT_MOC_LITERAL(51, 664, 12), // "CountryMusic"
QT_MOC_LITERAL(52, 677, 13), // "NationalMusic"
QT_MOC_LITERAL(53, 691, 11), // "OldiesMusic"
QT_MOC_LITERAL(54, 703, 9), // "FolkMusic"
QT_MOC_LITERAL(55, 713, 11), // "Documentary"
QT_MOC_LITERAL(56, 725, 9), // "AlarmTest"
QT_MOC_LITERAL(57, 735, 5), // "Alarm"
QT_MOC_LITERAL(58, 741, 4), // "Talk"
QT_MOC_LITERAL(59, 746, 11), // "ClassicRock"
QT_MOC_LITERAL(60, 758, 9), // "AdultHits"
QT_MOC_LITERAL(61, 768, 8), // "SoftRock"
QT_MOC_LITERAL(62, 777, 5), // "Top40"
QT_MOC_LITERAL(63, 783, 4), // "Soft"
QT_MOC_LITERAL(64, 788, 9), // "Nostalgia"
QT_MOC_LITERAL(65, 798, 9), // "Classical"
QT_MOC_LITERAL(66, 808, 14), // "RhythmAndBlues"
QT_MOC_LITERAL(67, 823, 18), // "SoftRhythmAndBlues"
QT_MOC_LITERAL(68, 842, 8), // "Language"
QT_MOC_LITERAL(69, 851, 14), // "ReligiousMusic"
QT_MOC_LITERAL(70, 866, 13), // "ReligiousTalk"
QT_MOC_LITERAL(71, 880, 11), // "Personality"
QT_MOC_LITERAL(72, 892, 6), // "Public"
QT_MOC_LITERAL(73, 899, 7) // "College"

    },
    "QRadioData\0stationIdChanged\0\0stationId\0"
    "programTypeChanged\0QRadioData::ProgramType\0"
    "programType\0programTypeNameChanged\0"
    "programTypeName\0stationNameChanged\0"
    "stationName\0radioTextChanged\0radioText\0"
    "alternativeFrequenciesEnabledChanged\0"
    "enabled\0error\0QRadioData::Error\0"
    "setAlternativeFrequenciesEnabled\0"
    "_q_serviceDestroyed\0ProgramType\0"
    "alternativeFrequenciesEnabled\0Error\0"
    "NoError\0ResourceError\0OpenError\0"
    "OutOfRangeError\0Undefined\0News\0"
    "CurrentAffairs\0Information\0Sport\0"
    "Education\0Drama\0Culture\0Science\0Varied\0"
    "PopMusic\0RockMusic\0EasyListening\0"
    "LightClassical\0SeriousClassical\0"
    "OtherMusic\0Weather\0Finance\0"
    "ChildrensProgrammes\0SocialAffairs\0"
    "Religion\0PhoneIn\0Travel\0Leisure\0"
    "JazzMusic\0CountryMusic\0NationalMusic\0"
    "OldiesMusic\0FolkMusic\0Documentary\0"
    "AlarmTest\0Alarm\0Talk\0ClassicRock\0"
    "AdultHits\0SoftRock\0Top40\0Soft\0Nostalgia\0"
    "Classical\0RhythmAndBlues\0SoftRhythmAndBlues\0"
    "Language\0ReligiousMusic\0ReligiousTalk\0"
    "Personality\0Public\0College"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QRadioData[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       6,   84, // properties
       2,  108, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,
       4,    1,   62,    2, 0x06 /* Public */,
       7,    1,   65,    2, 0x06 /* Public */,
       9,    1,   68,    2, 0x06 /* Public */,
      11,    1,   71,    2, 0x06 /* Public */,
      13,    1,   74,    2, 0x06 /* Public */,
      15,    1,   77,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      17,    1,   80,    2, 0x0a /* Public */,
      18,    0,   83,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void, 0x80000000 | 16,   15,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void,

 // properties: name, type, flags
       3, QMetaType::QString, 0x00495001,
       6, 0x80000000 | 19, 0x00495009,
       8, QMetaType::QString, 0x00495001,
      10, QMetaType::QString, 0x00495001,
      12, QMetaType::QString, 0x00495001,
      20, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,

 // enums: name, alias, flags, count, data
      21,   21, 0x0,    4,  118,
      19,   19, 0x0,   48,  126,

 // enum data: key, value
      22, uint(QRadioData::NoError),
      23, uint(QRadioData::ResourceError),
      24, uint(QRadioData::OpenError),
      25, uint(QRadioData::OutOfRangeError),
      26, uint(QRadioData::Undefined),
      27, uint(QRadioData::News),
      28, uint(QRadioData::CurrentAffairs),
      29, uint(QRadioData::Information),
      30, uint(QRadioData::Sport),
      31, uint(QRadioData::Education),
      32, uint(QRadioData::Drama),
      33, uint(QRadioData::Culture),
      34, uint(QRadioData::Science),
      35, uint(QRadioData::Varied),
      36, uint(QRadioData::PopMusic),
      37, uint(QRadioData::RockMusic),
      38, uint(QRadioData::EasyListening),
      39, uint(QRadioData::LightClassical),
      40, uint(QRadioData::SeriousClassical),
      41, uint(QRadioData::OtherMusic),
      42, uint(QRadioData::Weather),
      43, uint(QRadioData::Finance),
      44, uint(QRadioData::ChildrensProgrammes),
      45, uint(QRadioData::SocialAffairs),
      46, uint(QRadioData::Religion),
      47, uint(QRadioData::PhoneIn),
      48, uint(QRadioData::Travel),
      49, uint(QRadioData::Leisure),
      50, uint(QRadioData::JazzMusic),
      51, uint(QRadioData::CountryMusic),
      52, uint(QRadioData::NationalMusic),
      53, uint(QRadioData::OldiesMusic),
      54, uint(QRadioData::FolkMusic),
      55, uint(QRadioData::Documentary),
      56, uint(QRadioData::AlarmTest),
      57, uint(QRadioData::Alarm),
      58, uint(QRadioData::Talk),
      59, uint(QRadioData::ClassicRock),
      60, uint(QRadioData::AdultHits),
      61, uint(QRadioData::SoftRock),
      62, uint(QRadioData::Top40),
      63, uint(QRadioData::Soft),
      64, uint(QRadioData::Nostalgia),
      65, uint(QRadioData::Classical),
      66, uint(QRadioData::RhythmAndBlues),
      67, uint(QRadioData::SoftRhythmAndBlues),
      68, uint(QRadioData::Language),
      69, uint(QRadioData::ReligiousMusic),
      70, uint(QRadioData::ReligiousTalk),
      71, uint(QRadioData::Personality),
      72, uint(QRadioData::Public),
      73, uint(QRadioData::College),

       0        // eod
};

void QRadioData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QRadioData *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->stationIdChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->programTypeChanged((*reinterpret_cast< QRadioData::ProgramType(*)>(_a[1]))); break;
        case 2: _t->programTypeNameChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 3: _t->stationNameChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 4: _t->radioTextChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 5: _t->alternativeFrequenciesEnabledChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->error((*reinterpret_cast< QRadioData::Error(*)>(_a[1]))); break;
        case 7: _t->setAlternativeFrequenciesEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->d_func()->_q_serviceDestroyed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QRadioData::ProgramType >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QRadioData::Error >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QRadioData::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QRadioData::stationIdChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QRadioData::*)(QRadioData::ProgramType );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QRadioData::programTypeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QRadioData::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QRadioData::programTypeNameChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QRadioData::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QRadioData::stationNameChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QRadioData::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QRadioData::radioTextChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QRadioData::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QRadioData::alternativeFrequenciesEnabledChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QRadioData::*)(QRadioData::Error );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QRadioData::error)) {
                *result = 6;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QRadioData *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->stationId(); break;
        case 1: *reinterpret_cast< ProgramType*>(_v) = _t->programType(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->programTypeName(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->stationName(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->radioText(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->isAlternativeFrequenciesEnabled(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QRadioData *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 5: _t->setAlternativeFrequenciesEnabled(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QRadioData::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QRadioData.data,
    qt_meta_data_QRadioData,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QRadioData::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QRadioData::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QRadioData.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QMediaBindableInterface"))
        return static_cast< QMediaBindableInterface*>(this);
    if (!strcmp(_clname, "org.qt-project.qt.mediabindable/5.0"))
        return static_cast< QMediaBindableInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int QRadioData::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QRadioData::stationIdChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QRadioData::programTypeChanged(QRadioData::ProgramType _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QRadioData::programTypeNameChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QRadioData::stationNameChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QRadioData::radioTextChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QRadioData::alternativeFrequenciesEnabledChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QRadioData::error(QRadioData::Error _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
