/****************************************************************************
** Meta object code from reading C++ file 'camerabinresourcepolicy.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../git/src/plugins/gstreamer/camerabin/camerabinresourcepolicy.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'camerabinresourcepolicy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CamerabinResourcePolicy_t {
    QByteArrayData data[11];
    char stringdata0[193];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CamerabinResourcePolicy_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CamerabinResourcePolicy_t qt_meta_stringdata_CamerabinResourcePolicy = {
    {
QT_MOC_LITERAL(0, 0, 23), // "CamerabinResourcePolicy"
QT_MOC_LITERAL(1, 24, 15), // "resourcesDenied"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 16), // "resourcesGranted"
QT_MOC_LITERAL(4, 58, 13), // "resourcesLost"
QT_MOC_LITERAL(5, 72, 17), // "canCaptureChanged"
QT_MOC_LITERAL(6, 90, 19), // "handleResourcesLost"
QT_MOC_LITERAL(7, 110, 22), // "handleResourcesGranted"
QT_MOC_LITERAL(8, 133, 23), // "handleResourcesReleased"
QT_MOC_LITERAL(9, 157, 18), // "resourcesAvailable"
QT_MOC_LITERAL(10, 176, 16) // "updateCanCapture"

    },
    "CamerabinResourcePolicy\0resourcesDenied\0"
    "\0resourcesGranted\0resourcesLost\0"
    "canCaptureChanged\0handleResourcesLost\0"
    "handleResourcesGranted\0handleResourcesReleased\0"
    "resourcesAvailable\0updateCanCapture"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CamerabinResourcePolicy[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x06 /* Public */,
       3,    0,   60,    2, 0x06 /* Public */,
       4,    0,   61,    2, 0x06 /* Public */,
       5,    0,   62,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,   63,    2, 0x08 /* Private */,
       7,    0,   64,    2, 0x08 /* Private */,
       8,    0,   65,    2, 0x08 /* Private */,
       9,    0,   66,    2, 0x08 /* Private */,
      10,    0,   67,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void CamerabinResourcePolicy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CamerabinResourcePolicy *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->resourcesDenied(); break;
        case 1: _t->resourcesGranted(); break;
        case 2: _t->resourcesLost(); break;
        case 3: _t->canCaptureChanged(); break;
        case 4: _t->handleResourcesLost(); break;
        case 5: _t->handleResourcesGranted(); break;
        case 6: _t->handleResourcesReleased(); break;
        case 7: _t->resourcesAvailable(); break;
        case 8: _t->updateCanCapture(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CamerabinResourcePolicy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CamerabinResourcePolicy::resourcesDenied)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CamerabinResourcePolicy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CamerabinResourcePolicy::resourcesGranted)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CamerabinResourcePolicy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CamerabinResourcePolicy::resourcesLost)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CamerabinResourcePolicy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CamerabinResourcePolicy::canCaptureChanged)) {
                *result = 3;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject CamerabinResourcePolicy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_CamerabinResourcePolicy.data,
    qt_meta_data_CamerabinResourcePolicy,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CamerabinResourcePolicy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CamerabinResourcePolicy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CamerabinResourcePolicy.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CamerabinResourcePolicy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void CamerabinResourcePolicy::resourcesDenied()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CamerabinResourcePolicy::resourcesGranted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void CamerabinResourcePolicy::resourcesLost()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void CamerabinResourcePolicy::canCaptureChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
